@@ -21,8 +21,8 @@
 ;; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 ;; USA
 
-;; Package-Version: 20260217.1416
-;; Package-Revision: 28b5bbf1a026
+;; Package-Version: 20260223.1935
+;; Package-Revision: e2664f040eb7
 ;; Author: Adrien Brochard
 ;; Keywords: kubernetes k8s tools processes
 ;; URL: https://github.com/abrochard/kubel
@@ -352,10 +352,8 @@ If SILENCE-WARNINGS is non-nil, stderr output is not displayed."
 (defvar-local kubel-resource "pods"
   "Current resource.")
 
-(defvar-local kubel-context
-  (replace-regexp-in-string
-   "\n" "" (kubel--exec-sync "kubectl config current-context"))
-  "Current context.  Tries to smart default.")
+(defvar-local kubel-context nil
+  "Current context.  Initialized lazily on first use.")
 
 (defvar-local kubel-resource-filter ""
   "Substring filter for resource name.")
@@ -379,6 +377,10 @@ If SILENCE-WARNINGS is non-nil, stderr output is not displayed."
 
 (defvar-local kubel--kubernetes-resources-list-cached nil)
 
+(defvar-local kubel--parent-buffer nil
+  "The buffer that spawned this kubel buffer.
+Used for back-navigation when pressing q.")
+
 (defun kubel--kubernetes-resources-list ()
   "Get list of resources from cache or from fetching the api resource."
   (if (null kubel--kubernetes-resources-list-cached)
@@ -716,6 +718,27 @@ TYPENAME is the resource type/name."
 	    (yes-or-no-p "Resource modified; kill anyway? "))
     (kill-buffer (current-buffer))))
 
+(defun kubel-go-back ()
+  "Kill current buffer and return to parent kubel buffer.
+If no parent buffer exists, call `quit-window'."
+  (interactive)
+  (let ((parent kubel--parent-buffer))
+    (if (and parent (buffer-live-p parent))
+        (progn
+          (kill-buffer (current-buffer))
+          (switch-to-buffer parent)
+          ;; Re-display error overlay if there was an error
+          (when kubel--header-error
+            (kubel--set-header-error kubel--header-error)))
+      (quit-window))))
+
+(defun kubel-kill-all-buffers ()
+  "Kill all kubel-related buffers."
+  (interactive)
+  (dolist (buf (buffer-list))
+    (when (string-prefix-p "*kubel:" (buffer-name buf))
+      (kill-buffer buf))))
+
 (defvar kubel-yaml-editing-mode-map
   (let ((map (make-sparse-keymap)))
     (define-key map (kbd "C-c C-c") #'kubel-apply)
@@ -765,13 +788,8 @@ Allows simple apply of the changes made.
                      (set-buffer-modified-p nil)
                      (goto-char (point-min)))))
     (if describe
-        (kubel--exec-async process-name (list "describe" kubel-resource (kubel--get-resource-under-cursor)) nil callback)
-      (kubel--exec-async process-name (list "get" kubel-resource (kubel--get-resource-under-cursor) "-o" kubel-output) nil callback))
-    (when (or (string-equal kubel-output "yaml") (transient-args 'kubel-describe-popup))
-      (kubel-yaml-editing-mode)
-      (setq kubel-context ctx)
-      (setq kubel-namespace ns)
-      (setq kubel-resource res))))
+        (kubel--exec-async process-name (list "describe" kubel-resource (kubel--get-resource-under-cursor)) t callback)
+      (kubel--exec-async process-name (list "get" kubel-resource (kubel--get-resource-under-cursor) "-o" kubel-output) t callback))))
 
 (defun kubel--default-tail-arg (args)
   "Ugly function to make sure that there is at least the default tail.
@@ -901,9 +919,11 @@ the context caches, including the cached resource list."
   (let* ((namespace (completing-read "Namespace: " (kubel--list-namespace)
                                      nil nil nil nil "default"))
          (kubel--buffer (get-buffer (kubel--buffer-name)))
+         (parent-buffer (current-buffer))
          (last-default-directory (when kubel--buffer
                                    (with-current-buffer kubel--buffer default-directory))))
     (with-current-buffer (clone-buffer)
+      (setq kubel--parent-buffer parent-buffer)
       (setq kubel-namespace namespace)
       (kubel--add-namespace-to-history namespace)
       (switch-to-buffer (current-buffer))
@@ -913,8 +933,10 @@ the context caches, including the cached resource list."
   "Set the context."
   (interactive)
   (let* ((kubel--buffer (get-buffer (kubel--buffer-name)))
+         (parent-buffer (current-buffer))
          (last-default-directory (when kubel--buffer (with-current-buffer kubel--buffer default-directory))))
     (with-current-buffer (clone-buffer)
+      (setq kubel--parent-buffer parent-buffer)
       (setq kubel-context
             (completing-read
              "Select context: "
@@ -945,17 +967,19 @@ the context caches, including the cached resource list."
 (defun kubel-set-label-selector ()
   "Set the selector."
   (interactive)
-  (with-current-buffer (clone-buffer)
-    (let ((selector (completing-read
+  (let ((parent-buffer (current-buffer)))
+    (with-current-buffer (clone-buffer)
+      (setq kubel--parent-buffer parent-buffer)
+      (let ((selector (completing-read
                      "Selector: "
                      (kubel--list-selectors))))
       (when (equal selector "none")
         (setq selector ""))
       (setq kubel-selector selector))
-    (kubel--add-selector-to-history kubel-selector)
-    ;; Update pod list according to the label selector
-    (switch-to-buffer (current-buffer))
-    (kubel-refresh)))
+      (kubel--add-selector-to-history kubel-selector)
+      ;; Update pod list according to the label selector
+      (switch-to-buffer (current-buffer))
+      (kubel-refresh))))
 
 (defun kubel--fetch-api-resource-list ()
   "Fetch the API resource list."
@@ -971,8 +995,10 @@ the context caches, including the cached resource list."
   (let* ((current-buffer-name (kubel--buffer-name))
          (resource-list (kubel--kubernetes-resources-list))
          (kubel--buffer (get-buffer current-buffer-name))
+         (parent-buffer (current-buffer))
          (last-default-directory (when kubel--buffer (with-current-buffer kubel--buffer default-directory))))
     (with-current-buffer (clone-buffer)
+      (setq kubel--parent-buffer parent-buffer)
       (setq kubel-resource
             (completing-read "Select resource: " resource-list))
       (switch-to-buffer (current-buffer))
@@ -1409,6 +1435,9 @@ When called interactively, prompts for a buffer belonging to kubel."
     (define-key map (kbd "M") 'kubel-mark-all)
     (define-key map (kbd "U") 'kubel-unmark-all)
 
+    (define-key map (kbd "q") 'kubel-go-back)
+    (define-key map (kbd "Q") 'kubel-kill-all-buffers)
+
     map)
   "Keymap for `kubel-mode'.")
 
@@ -1467,6 +1496,15 @@ DIRECTORY is optional for TRAMP support."
         (pop-to-buffer-same-window tmpname)
         (kubel-refresh directory)))))
 
+(defun kubel--init-context ()
+  "Initialize context and namespace if not already set.
+Defers the kubectl call until first use to avoid blocking Emacs startup."
+  (unless kubel-context
+    (setq kubel-context
+          (replace-regexp-in-string
+           "\n" "" (kubel--exec-sync "kubectl config current-context")))
+    (setq kubel-namespace kubel-default-namespace)))
+
 ;;;###autoload
 (defun kubel (&optional directory)
   "Invoke the kubel buffer.
@@ -1479,6 +1517,7 @@ DIRECTORY is optional for TRAMP support."
       (switch-to-buffer (current-buffer))
       (unless (eq major-mode 'kubel-mode)
         (kubel-mode))
+      (kubel--init-context)
       (kubel-refresh directory))))
 
 (define-derived-mode kubel-mode tabulated-list-mode "Kubel"
